game.module(
    'game.class.timerHudTraining'
)

game.body(function () {

    game.createClass('ClassTimerHudTraining', {
        graphicsContainer: {},     // graficke prvky timeru
        spriteBackground: {},     // pozadi celeho timeru
        timeBackground: {},     // casova linka
        levelNumber: '',

        remainingPatterns: 0,

        timeMaxWidth: 0,        // max width casove osy
        defaultTime: 0,     // puvodni max cas. slouzi pro zmensovani casove osy
        remainingTime: 0,	   // cas ktery zbyva do konce
        time: 0,        // time in ms
        displayTime: '',       // cas pro zobrazeni
        displayRemainingTime: '',       // zbyvajici cas pro zobrazeni

        init: function (levelNumber, remainingTime) {
            this.levelNumber = levelNumber;
            this.setTimer(remainingTime);
            this.createGraphics();
        },

        createGraphics: function () {
            // vytvori sprites (grafiku) timeru
            this.graphicsContainer = new game.Container();

            this.spriteTimeBackground = new game.Sprite('images/timer-time-bg.png');
            this.spriteTimeBackground.position.set((game.system.width / 2), 138);
            this.spriteTimeBackground.anchor.set(0.5, 0.5);
            this.graphicsContainer.addChild(this.spriteTimeBackground);

            this.spriteTime = new game.Sprite('images/timer-time.png');
            this.spriteTime.position.set((game.system.width / 2) - 320, 115);
            this.spriteTime.alpha = 0;
            //this.spriteTime.anchor.set(0.5, 0.5);
            this.graphicsContainer.addChild(this.spriteTime);

            this.timeMaxWidth = this.spriteTime.width;

            this.spriteBackground = new game.Sprite('images/timer-bg.png');
            this.spriteBackground.position.set((game.system.width / 2), 138);
            this.spriteBackground.anchor.set(0.5, 0.5);
            this.graphicsContainer.addChild(this.spriteBackground);

            this.textLevel = new game.BitmapText('Level ' + this.levelNumber, {font: '50px level-text'});
            this.textLevel.position.set( (game.system.width / 2) - (this.textLevel.width / 2), 25);
            this.graphicsContainer.addChild(this.textLevel);

            this.textTime = new game.BitmapText(this.displayRemainingTime, {font: '50px level-text'});
            this.textTime.position.set((game.system.width / 2) - 330, 25);      // 155
            this.graphicsContainer.addChild(this.textTime);

            this.textPatternsNumber = new game.BitmapText(this.remainingPatterns.toString(), {font: '60px pattern-text'});
            this.textPatternsNumber.position.set((game.system.width / 2) + 269 - (this.textPatternsNumber.textWidth  / 2), 114);
            this.graphicsContainer.addChild(this.textPatternsNumber);
        },

        setTimer: function (remainingTime) {
            this.defaultTime = remainingTime;
            this.remainingTime = remainingTime;

            var resultTime = this.convertTime(this.remainingTime, true);
            this.displayRemainingTime = resultTime.min.toString() + ':' + resultTime.sec.toString();

            var resultTime = this.convertTime(this.time, true);
            this.displayTime = resultTime.min.toString() + ':' + resultTime.sec.toString();
        },

        showTimeLine: function () {
            var tween = new game.Tween(this.spriteTime)
                .to({alpha: 1}, 100)
                .start();
        },

        convertTime: function (time, zero) {
            // prevod casu na minuty a vteriny
            // zero tru pokud pridat nuly jinak false
            var min = 0;
            var sec = 0;
            var ms = 0;

            if (time >= 60000) {
                min = Math.floor(time / 60000);
                rest = time % 60000;
                if (rest > 0) {
                    sec = Math.ceil(rest / 1000);
                }
            } else {
                min = 0;
                sec = Math.ceil(time / 1000);
            }

            if (sec <= 9 && zero == true) {
                sec = '0' + sec;
            }

            var result = {};

            result.min = min.toString();
            result.sec = sec.toString();

            return result;
        },

        updateTime: function () {
            this.remainingTime -= 100;
            this.time += 100;

            var resultTime = this.convertTime(this.remainingTime, true);
            this.displayRemainingTime = resultTime.min.toString() + ':' + resultTime.sec.toString();

            var resultTime = this.convertTime(this.time, true);
            this.displayTime = resultTime.min.toString() + ':' + resultTime.sec.toString();

            this.textTime.setText(this.displayRemainingTime);
            //console.log(this.displayTime);

            // zmensuje casovou osu
            var ratio = (this.defaultTime / this.remainingTime);
            var newWidth = Math.round(this.timeMaxWidth / ratio);

            var tween = new game.Tween(this.spriteTime)
                .to({width: newWidth}, 100)
                .start();

            //this.spriteTime.width = newWidth;


            if (this.remainingTime <= 0) {
                return false;
            } else {
                return true;
            }
        },

        reduceRemainingPatterns: function () {
            this.remainingPatterns -= 1;
            // kdyz chci centrovat text musim text odebrat, znovu vytvorit a plozit do sceny. BUG pixi.js po setText ma prodlevu nez aktualizuje width a textWidth... text tak neni aktualne vycentrovany
            this.graphicsContainer.removeChild(this.textPatternsNumber);
            this.textPatternsNumber = new game.BitmapText(this.remainingPatterns.toString(), {font: '60px pattern-text'});
            this.textPatternsNumber.position.set((game.system.width / 2) + 269 - (this.textPatternsNumber.textWidth  / 2), 114);
            this.graphicsContainer.addChild(this.textPatternsNumber);
        },

        incrementRemainingPatterns: function () {
            this.remainingPatterns += 1;
            // kdyz chci centrovat text musim text odebrat, znovu vytvorit a plozit do sceny. BUG pixi.js po setText ma prodlevu nez aktualizuje width a textWidth... text tak neni aktualne vycentrovany
            this.graphicsContainer.removeChild(this.textPatternsNumber);
            this.textPatternsNumber = new game.BitmapText(this.remainingPatterns.toString(), {font: '60px pattern-text'});
            this.textPatternsNumber.position.set((game.system.width / 2) + 269 - (this.textPatternsNumber.textWidth  / 2), 114);
            this.graphicsContainer.addChild(this.textPatternsNumber);
        },

        returnTime: function () {
            return this.time;
        },
        returnRemainingTime: function () {
            return this.remainingTime;
        },

        returnDisplayTime: function () {
            return this.displayTime;
        },
        returnDisplayRemainingTime: function () {
            return this.displayRemainingTime;
        },



    });

});
