game.module(
    'game.scenes.gameChallenge'
);

game.body(function() {

    game.createScene('GameChallenge', {
        backgroundColor: 0xb9bec7,      // 0xb9bec7     0x3f4144        0xb8babf
        shapesAndColors: [],      // kombinace povolenych tvaru a k nim zbyvajici barvy
        locations: [],      // pole pozic vsech tvaru
        paterns: [],    // pole pozic prikladu
        remainingPatterns: 0,
        levelPoints: 0,
        timerHud: {},  // objekt timer HUD
        score: {},         // objekt vysledneho score
        levelCompleted: true,       // pokud win tak true pokud nebzl level dokoncen false
        nextLevelExist: false,      // existuje dalsi level ?
        nextLevelId: 0,     // id dalsiho levelu - pokud existuje

        init: function() {
            // Google Analytics
            if (navigator.isCocoonJS) {
               game.Analytics.ga.trackView('Game - level ' + game.LevelChallenge.levelNumber + ' (Challenge)');
            }

            this.levelTime = game.LevelChallenge.levelTime;
            this.remainingPatterns = game.LevelChallenge.patternNumber;

            this.addBackground();
            this.addHud();
            this.prepareShapes();

            //console.log('level name: ' + game.LevelChallenge.name);
            //console.log('level locationsNumber: ' + game.LevelChallenge.locationsNumber);
            //console.log('level levelTime: ' + this.levelTime);
            //console.log('level remainingPatterns: ' + this.remainingPatterns);

        },

        addBackground: function () {
            var background = new game.Sprite('images/game-background.png');
            background.position.set((game.system.width / 2), 0);
            background.anchor.set(0.5, 0);
            background.addTo(this.stage);
            /*
            var banner = new game.Sprite('images/banner.png');
            banner.position.set((game.system.width / 2), game.system.height - 100);
            banner.anchor.set(0.5, 0);
            banner.addTo(this.stage);
            */
        },

        addHud: function() {
            var self = this;
            var menuBack = new game.Sprite('images/btn-menu-back.png');
            menuBack.position.set(((game.system.width/2) - 330 + (menuBack.width / 2)), 1178);
            menuBack.anchor.set(0.5, 0.5);
            menuBack.addTo(this.stage);

            menuBack.interactive = true;
            menuBack.click = menuBack.tap = function() {
                menuBack.interactive = false;

                var tween = new game.Tween(menuBack.scale)
                        .to({x: 0.75, y: 0.75}, 200)
                        .repeat(1)
                        .yoyo()
                        .easing(game.Tween.Easing.Quadratic.Out)
                        .onComplete(function () {
                            menuBack.interactive = true;
                            game.system.setScene('MainMenu');
                        });

                game.audio.playSound('button-click');
                tween.start();
            };


            var restart = new game.Sprite('images/btn-restart.png');
            restart.position.set((game.system.width/2), 1178);
            restart.anchor.set(0.5, 0.5);
            restart.addTo(this.stage);

            restart.interactive = true;
            restart.click = restart.tap = function() {
                restart.interactive = false;

                game.challengeTotalPoints = 0;
                game.LevelChallenge.setLevel(1);

                var tween = new game.Tween(restart.scale)
                        .to({x: 0.75, y: 0.75}, 200)
                        .repeat(1)
                        .yoyo()
                        .easing(game.Tween.Easing.Quadratic.Out)
                        .onComplete(function () {
                            restart.interactive = true;
                            game.system.setScene('GameChallenge');
                        });

                // Google Analytics
                if (navigator.isCocoonJS) {
                    game.Analytics.ga.trackEvent('Game actions', 'Restart game - level ' + game.LevelChallenge.levelNumber + ' (Challenge)');
                }

                game.audio.playSound('button-click');
                tween.start();

            };

            this.timerHud = new game.ClassTimerHudTraining(game.LevelChallenge.levelNumber, game.LevelChallenge.levelTime);
            this.timerHud.graphicsContainer.addTo(this.stage);

        },

        prepareShapes: function() {
            var self = this;

            // jednotlive nazvy tvaru
            var shapesArray = [];
            var index = 0;
            for (var i = 0; i < game.Shapes.shapesNumber; i += 1) {
                index = i + 1;
                shapesArray[i] = game.Shapes['shape' + index].name;
            }

            // nahodne promichani tvaru
            var shapesTempArray = Object.keys(shapesArray);
            var shapesRandomArray = [];
            for (var i = 0; i < shapesArray.length; i += 1) {
                var randomIndexShape = Math.round( (Math.random() * (shapesTempArray.length - 1)) );
                var shapeKey = shapesTempArray[randomIndexShape];
                shapesRandomArray[i] = shapesArray[shapeKey];
                shapesTempArray.splice(randomIndexShape, 1);
            }


            // jednotlive nazvy barev
            var colorsArray = [];
            var index = 0;
            for (var i = 0; i < game.Colors.colorsNumber; i += 1) {
                index = i + 1;
                colorsArray[i] = game.Colors['color' + index].name;
            }

            // nahodne promichani barev
            var colorsTempArray = Object.keys(colorsArray);
            var colorsRandomArray = [];
            for (var i = 0; i < colorsArray.length; i += 1) {
                var randomIndexColor = Math.round( (Math.random() * (colorsTempArray.length - 1)) );
                var colorKey = colorsTempArray[randomIndexColor];
                colorsRandomArray[i] = colorsArray[colorKey];
                colorsTempArray.splice(randomIndexColor, 1);
            }


            // sestavim pole tvaru a urcim kolikrat se musi tvar opakovat aby jich bylo dost na vsechny pozice v levelu - serazeneho od nejvetsi quantity
            var index = 0;
            var shapesQuantityArray = [];
            for (var i = 0; i < game.LevelChallenge.locationsNumber; i += 1) {
                if (index >= shapesRandomArray.length) {
                    index = 0;
                }

                if (shapesQuantityArray[index] == undefined) {
                    shapesQuantityArray[index] = {};
                    shapesQuantityArray[index].name = shapesRandomArray[index];
                    shapesQuantityArray[index].quantity = 1;
                } else {
                    shapesQuantityArray[index].quantity += 1;
                }

                index += 1;
            }

            // sestavim pole barev a urcim kolikrat se musi barva opakovat aby jich bylo dost na vsechny pozice v levelu - serazeneho od nejvetsi quantity
            var index = 0;
            var colorsQuantityArray = [];
            for (var i = 0; i < game.LevelChallenge.locationsNumber; i += 1) {
                if (index >= colorsRandomArray.length) {
                    index = 0;
                }

                if (colorsQuantityArray[index] == undefined) {
                    colorsQuantityArray[index] = {};
                    colorsQuantityArray[index].name = colorsRandomArray[index];
                    colorsQuantityArray[index].quantity = 1;
                } else {
                    colorsQuantityArray[index].quantity += 1;
                }

                index += 1;
            }


            // sestavim pole finalnich kombinaci
            var index = 0;
            for (var i = 0; i < shapesQuantityArray.length; i += 1) {
                // sestavi pole barev povolene pro tvar
                var colorsForShape = [];
                var indexColor = 0;
                for (var f = 0; f < colorsQuantityArray.length; f += 1) {
                    if (colorsQuantityArray[f].quantity > 0) {
                        colorsForShape[indexColor] = {};
                        colorsForShape[indexColor].name = colorsQuantityArray[f].name;
                        colorsForShape[indexColor].origColorIndex = f;

                        indexColor += 1;
                    }
                }


                // pro kazdy tvar probehne tolikrat kolik je quantity
                for (var e = 0; e < shapesQuantityArray[i].quantity; e += 1) {
                    // vyneru barvu z povolenych barev pro jednotlivy tvar - postupne od barvy s nejvetsim quantity
                    var color = colorsForShape[e].name;
                    var removedIndex = colorsForShape[e].origColorIndex;

                    // odebrani barvy ze vsech dostupnych barev
                    colorsQuantityArray[removedIndex].quantity -= 1;

                    this.shapesAndColors[index] = shapesQuantityArray[i].name + '-' + color;

                    index += 1;
                }

                // po snizeni poctu barev pole setridit podle barev s nejvetsi quantity
                colorsQuantityArray.sort(function(a, b){
                    return b.quantity - a.quantity;
                })



            }

            /*
            console.log('this.shapesAndColors: ');
            console.log(this.shapesAndColors);
            */


            this.prepareLocations();
        },

        prepareLocations: function() {
            var randomId, shape;
            var self = this;

            for (var i = 0; i < game.LevelChallenge.locationsNumber; i += 1) {
                var randomIndex = Math.round( (Math.random() * (this.shapesAndColors.length - 1)) );

                //console.log('images/shape-' + this.shapesAndColors[randomIndex] + '.png');

                ///*
                this.locations[i] = {};
                // game.LevelChallenge.locationsSprites[i]
                var posX = (game.system.width/2);
                var posY = 240 + Math.round((game.LevelChallenge.locationSize) / 2);

                this.locations[i].sprite = new game.Sprite('images/shape-' + this.shapesAndColors[randomIndex] + '.png');
                this.locations[i].sprite.width = game.LevelChallenge.shapeSize;
                this.locations[i].sprite.height = game.LevelChallenge.shapeSize;
                this.locations[i].sprite.position.set(posX + (game.LevelChallenge.locations[i].x), posY + game.LevelChallenge.locations[i].y);
                this.locations[i].sprite.anchor.set(0.5, 0.5);
                this.locations[i].sprite.alpha = 1;
                this.locations[i].sprite.addTo(this.stage);
                this.locations[i].isPattern = false;        // tvar je vzor
                this.locations[i].isHidden = false;     //  urcuje jestli je tvar skryty nebo ne. pouzivam pro urceni ktery tvar se ma pri zaverecne animaci zmizeni zbylych tvaru presunout.
                //*/
                this.shapesAndColors.splice(randomIndex, 1);
            }

            this.preparePattern();
        },

        preparePattern: function () {
            var self = this;
            var randomIndex2, randomKey2;
            // vsechny klice z pole
            var locationsKeys = Object.keys(this.locations);
            //console.log('locationsKeys: ' + locationsKeys);

            for (var i = 0; i < game.LevelChallenge.patternNumber; i += 1) {
                randomIndex2 = Math.round( (Math.random() * (locationsKeys.length - 1)) );
                this.locations[locationsKeys[randomIndex2]].isPattern = true;       // nastavi lokations na true kde je pattern
                this.paterns[i] = locationsKeys[randomIndex2];

                // odstraneni klice aby se nevylosoval stejny znovu
                locationsKeys.splice(randomIndex2, 1);
                //console.log('locationsKeys: ' + locationsKeys);
            }

            this.startLevel();
        },

        addInteractiveForLocations: function (i) {
            var self = this;

            this.locations[i].sprite.interactive = true;
            this.locations[i].sprite.mousedown = this.locations[i].sprite.touchstart = function() {
                self.locations[i].sprite.interactive = false;
                self.checkPattern(i);
            };
        },

        removeInteractiveForLocations: function (i) {
            //console.log('index: ' + i);
            this.locations[i].sprite.interactive = false;
        },

        removeAllInteractiveForLocations: function () {
            for (var i = 0; i < game.LevelChallenge.locationsNumber; i += 1) {
                this.locations[i].sprite.interactive = false;
            }
        },

        startLevel: function () {
            // Google Analytics
            if (navigator.isCocoonJS) {
                game.Analytics.ga.trackEvent('Game actions', 'Start game - level ' + game.LevelChallenge.levelNumber + ' (Challenge)');
            }

            // cachovani reklam
            if (window.cordova && game.Ads.isTimeForShowAds === true) {
                game.Ads.cacheAds();
            }


            var self = this;
            // spozdeni
            this.addTimer(1000, function () {
                self.startCountDown();
            });

        },

        startCountDown: function () {
            var self = this;
            var text = game.LevelChallenge.name;
            var positionY = 200;
            this.countDownText = new game.BitmapText(text, {font: '120px level-start'});
            this.countDownText.position.set((game.system.width / 2) - (this.countDownText.width / 2), (game.system.height / 2) - positionY - (this.countDownText.height / 2));
            this.countDownText.alpha = 0;
            this.countDownText.scale.set(0, 0);
            this.countDownText.addTo(this.stage);

            var tween1 = new game.Tween(this.countDownText.scale)
                .to({x: 1, y: 1}, 600)
                .easing(game.Tween.Easing.Cubic.Out)
                .onUpdate(function () {
                    self.countDownText.position.set((game.system.width / 2) - (self.countDownText.width / 2), (game.system.height / 2) - positionY - (self.countDownText.height / 2));
                });

            var tween2 = new game.Tween(this.countDownText.scale)
                .to({x: 0, y: 0}, 600)
                .easing(game.Tween.Easing.Cubic.In)
                .onUpdate(function () {
                    self.countDownText.position.set((game.system.width / 2) - (self.countDownText.width / 2), (game.system.height / 2) - positionY - (self.countDownText.height / 2));
                })
                .onComplete(function () {
                    self.stage.removeChild(self.countDownText);
                    self.startPattern();
                });


            var tween3 = new game.Tween(this.countDownText)
                .to({alpha: 1}, 600)
                .easing(game.Tween.Easing.Cubic.Out);

            var tween4 = new game.Tween(this.countDownText)
                .to({alpha: 0}, 600)
                .easing(game.Tween.Easing.Cubic.In);

            tween1.chain(tween2);
            tween1.start();

            tween3.chain(tween4);
            tween3.start();

        },

        startPattern: function () {
            var self = this;
            var delay = 1000;

            var origSize = game.LevelChallenge.shapeSize;
            var scaleSize = Math.round(game.LevelChallenge.shapeSize * 1.75);

            for (var i = 0; i < game.LevelChallenge.patternNumber; i += 1) {
                var index2 = this.paterns[i];

                if (game.Cheats === true) {
                    this.locations[index2].sprite.alpha = 0.25;
                }

                var tween1 = new game.Tween(this.locations[index2].sprite)
                    .to({width: scaleSize, height: scaleSize}, game.LevelChallenge.patternSpeed)
                    .easing(game.Tween.Easing.Quadratic.Out)
                    .onStart(function () {
                        game.audio.playSound('shape-click');
                    })
                    .onComplete(function () {
                        self.timerHud.incrementRemainingPatterns();
                    })
                    .delay(delay);
                //.easing(game.Tween.Easing.Quadratic.Out);


                var tween2 = new game.Tween(this.locations[index2].sprite)
                    .to({width: origSize, height: origSize}, game.LevelChallenge.patternSpeed)
                    .easing(game.Tween.Easing.Quadratic.In);
                //.easing(game.Tween.Easing.Quadratic.In);

                if (i == (game.LevelChallenge.patternNumber - 1)) {
                    tween2.onComplete(function () {
                        self.locations[index2].sprite.interactive = true;

                        // zpozdeni pred startem hry
                        self.addTimer(game.LevelChallenge.patternSpeed, function () {
                            self.startPlayerGame();
                        });
                    });
                } else {
                    tween2.onComplete(function () {
                        self.locations[index2].sprite.interactive = true;
                    });
                }

                tween1.chain(tween2);
                tween1.start();


                delay += game.LevelChallenge.patternDelay;
            }

            //console.log('game.LevelChallenge.patternSpeed: ' + game.LevelChallenge.patternSpeed);
            //console.log('game.LevelChallenge.patternDelay: ' + game.LevelChallenge.patternDelay);

            var difference = (game.LevelChallenge.patternDelay - (2 * game.LevelChallenge.patternSpeed));
            //console.log('difference: ' + difference);
        },

        startPlayerGame: function () {
            // povoli klikani na vsechny tvary

            for (var i = 0; i < game.LevelChallenge.locationsNumber; i += 1) {
                this.addInteractiveForLocations(i);
            }

            var self = this;

            // spusti odpocet casu
            this.timerHud.showTimeLine();
            this.addTimer(100, function () {
                self.addTimer(100, self.timerUpdate.bind(self), true);
            });

        },

        timerUpdate: function() {
            //console.log('TIMER Update');

            // spusti funkci na odpocet casu
            var check = this.timerHud.updateTime();
            if (check == false) {
                this.levelFailed();
            }

        },

        checkPattern: function (i) {
            // po kliknuti urci jestli se hrac trefil nebo ne.
            var self = this;

            var origSize = game.LevelChallenge.shapeSize;
            var scaleSize = Math.round(game.LevelChallenge.shapeSize * 1.75);

            this.removeInteractiveForLocations(i);

            if (this.locations[i].isPattern == true) {
                // spravne
                this.levelPoints += 1;

                // Google Analytics
                if (navigator.isCocoonJS) {
                    game.Analytics.ga.trackEvent('Game actions', 'Player got point - level ' + game.LevelChallenge.levelNumber + ' (Challenge)');
                }

                game.audio.playSound('shape-click');

                this.reduceRemainingPatterns();

                this.locations[i].sprite.setTexture('images/shape-check.png');
                this.locations[i].sprite.alpha = 0;
                this.locations[i].sprite.width = 0;
                this.locations[i].sprite.height = 0;

                var tween1 = new game.Tween(this.locations[i].sprite)
                .to({width: scaleSize, height: scaleSize, alpha: 1}, game.LevelChallenge.clickAnimationSpeed)
                .easing(game.Tween.Easing.Quadratic.Out)
                .onComplete(function () {
                    self.timerHud.reduceRemainingPatterns();
                });

                var tween2 = new game.Tween(this.locations[i].sprite)
                .easing(game.Tween.Easing.Quadratic.In)
                .to({width: origSize, height: origSize}, game.LevelChallenge.clickAnimationSpeed);


                tween1.chain(tween2);
                tween1.start();

            } else {
                // spatne
                game.audio.playSound('shape-pop');

                self.locations[i].isHidden = true;

                var tween1 = new game.Tween(this.locations[i].sprite)
                    .easing(game.Tween.Easing.Quadratic.Out)
                    .to({width: 0, height: 0, alpha: 0}, game.LevelChallenge.clickAnimationSpeed)
                    .onComplete(function () {
                        // chyba - konec vyzvy
                        self.levelFailed();
                    });

                tween1.start();
            }
        },

        reduceRemainingPatterns: function () {
            // snizi pocet zbyvajicich tvaru na uhodnuti
	       this.remainingPatterns -= 1;
           this.checkLevelEnd();
        },

        checkLevelEnd: function () {
            // zkontroluje kolik zbyva zivotu a pripadne ukonci level
            //console.log('this.remainingPatterns: ' + this.remainingPatterns);

            if (this.remainingPatterns <= 0) {
                this.levelWon();
            }
        },

        stopTimer: function () {
            //console.log('stop timer');
            this.removeTimer(this.timers[0]);
        },

        levelFailed: function () {
            // konec levelu - nesplneno

            // Google Analytics
            if (navigator.isCocoonJS) {
                game.Analytics.ga.trackEvent('Game actions', 'End game - Failed - level ' + game.LevelChallenge.levelNumber + ' (Challenge)');
            }

            this.levelCompleted = false;

            if (window.cordova) {
                game.Ads.wasAdsShown = false;
            }

            var self = this;

            //console.log('LEVEL FAILED');

            this.removeAllInteractiveForLocations();
            this.stopTimer();
            this.calculateScore();

            this.addTimer(1000, function () {
                self.hideAllLocations();
            });
        },

        levelWon: function () {
            // konec levelu - splneno

            // Google Analytics
            if (navigator.isCocoonJS) {
                game.Analytics.ga.trackEvent('Game actions', 'End game - Won - level ' + game.LevelChallenge.levelNumber + ' (Challenge)');
            }

            this.levelCompleted = true;

            if (window.cordova) {
                game.Ads.wasAdsShown = false;
            }

            var self = this;

            //console.log('LEVEL WON');

            this.removeAllInteractiveForLocations();
            this.stopTimer();
            this.calculateScore();

            this.addTimer(1000, function () {
                self.hideAllLocations();
            });
        },

        calculateScore: function () {
            // vypocita score udaje a ulozi je

            var remainingTime = this.timerHud.returnRemainingTime();
            var time = this.timerHud.returnTime();

            this.score.displayTime = this.timerHud.returnDisplayTime();
            this.score.levelPoints = this.levelPoints;

            this.score.totalPointsOld = game.challengeTotalPoints;
            this.score.totalPoints = game.challengeTotalPoints + this.score.levelPoints;

            game.challengeTotalPoints = game.challengeTotalPoints + this.score.levelPoints;

            //console.log('game.challengeTotalPoints: ' + game.challengeTotalPoints);
            //console.log('totalPointsOld: ' + this.score.totalPointsOld);
            //console.log('totalPoints: ' + this.score.totalPoints);

            // ulozeni score
            var storageKeyBestScore = 'levelChallengeBestScore';
            if (game.storage.has(storageKeyBestScore)) {
                this.score.best = game.storage.get(storageKeyBestScore);
            } else {
                this.score.best = 0;
            }

            // ulozeni best score
            if (this.score.totalPoints > this.score.best) {
                game.storage.set(storageKeyBestScore, this.score.totalPoints);
            }


            if (this.levelCompleted == true) {
                this.score.success = 100;
            } else {
                this.score.success = 0;
            }


            // ulozeni Unlocked pro dalsi level - odemknuti
            var nextLevel = 'level' + (game.LevelChallenge.levelNumber + 1);
            if (typeof game.LevelsChallenge.levels[nextLevel] !== "undefined") {
                // level existuje
                this.nextLevelExist = true;
                this.nextLevelId = (game.LevelChallenge.levelNumber + 1);
                //console.log('nextLevelId: ' + this.nextLevelId);
            }



            //console.log('score.displayTime: ' + this.score.displayTime);
            //console.log('score.success: ' + this.score.success + '%');
            //console.log('score.points: ' + this.score.points);
        },

        hideAllLocations: function () {
            var self = this;
            var delay = 0;
            var increment = 100 - Math.round(game.LevelChallenge.locationsNumber / 2);
            if (increment < 0) {
                increment = 10;
            }

            for (var i = 0; i < this.locations.length; i += 1) {
                if (this.locations[i].isHidden === false) {

                    var tween1 = new game.Tween(this.locations[i].sprite.position)
                        .to({x: (game.system.width / 2), y: ((game.system.height / 2) - 100)}, 500)
                        .delay(delay)
                        .easing(game.Tween.Easing.Quadratic.Out)
                        .start();

                    var tween2 = new game.Tween(this.locations[i].sprite)
                            .to({alpha: 0}, 300)
                            .delay(delay + 100)
                            //.easing(game.Tween.Easing.Quadratic.In)
                            .start();

                    delay += increment;
                }
            }

            this.addTimer(delay + 700, function () {
                self.showScore();
            });



        },

        showScore: function () {
            var self = this;

            self.addTimer(200, function() {
                // zobrazeni reklam

                if (window.cordova && game.Ads.isAdsReady === true && game.Ads.wasAdsShown === false) {
                    game.Ads.showAds();
                    game.Ads.wasAdsShown = true;        // na false se nastavuje ve funkci levelFailed a levelWon
                }

            }, false);

            var textTime = new game.BitmapText('Time: ' + this.score.displayTime.toString(), {font: '60px score-text'});
            textTime.position.set( ((game.system.width / 2) - (textTime.textWidth / 2)), 330);
            textTime.alpha = 0;
            textTime.addTo(this.stage);

            var textLevelPoints = new game.BitmapText('Level score: ' + this.score.levelPoints, {font: '60px score-text'});
            textLevelPoints.position.set( ((game.system.width / 2) - (textLevelPoints.textWidth / 2)), 410);
            textLevelPoints.alpha = 0;
            textLevelPoints.addTo(this.stage);

            var textTotalPoints = new game.BitmapText('Total score: ' + this.score.totalPointsOld, {font: '60px score-text'});
            textTotalPoints.position.set( ((game.system.width / 2) - (textTotalPoints.textWidth / 2)), 490);
            textTotalPoints.alpha = 0;
            textTotalPoints.addTo(this.stage);

            var textBest = new game.BitmapText('Best score: ' + this.score.best, {font: '60px score-text'});
            textBest.position.set( ((game.system.width / 2) - (textBest.textWidth / 2)), 610);
            textBest.alpha = 0;
            textBest.addTo(this.stage);

            //var success = this.score.success.toString() + '%';
            if (this.score.success > 0) {
                var textSuc = 'OK';
            } else {
                var textSuc = 'FAIL';
            }

            var textSuccessValue = new game.BitmapText(textSuc, {font: '120px score-success'});
            textSuccessValue.position.set( ((game.system.width / 2) - (textSuccessValue.textWidth / 2)), 760);
            textSuccessValue.alpha = 0;
            textSuccessValue.scale.set(0, 0);
            textSuccessValue.addTo(this.stage);


            var tweenTime = new game.Tween(textTime)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();

            var tweenMistakes = new game.Tween(textLevelPoints)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();

            var tweenMistakes = new game.Tween(textTotalPoints)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();

            var tweenBest = new game.Tween(textBest)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();


            var tweenSuccess1 = new game.Tween(textSuccessValue)
                    .to({alpha: 1}, 500)
                    .easing(game.Tween.Easing.Quadratic.Out);

            var tweenSuccess2 = new game.Tween(textSuccessValue.scale)
                    .to({x: 1, y: 1}, 500)
                    .easing(game.Tween.Easing.Quadratic.Out)
                    .onUpdate(function () {
                        var newPositionX = ((game.system.width / 2) - (textSuccessValue.width / 2));
                        textSuccessValue.position.set(newPositionX, textSuccessValue.position.y);
                    })
                    .onComplete(function () {
                        if (self.nextLevelExist == true && (self.score.success >= 50 || game.Cheats === true)) {       // 50%
                            var nextLevel = new game.Sprite('images/btn-next-level.png');
                            var positionX = (game.system.width / 2) + 330 - (nextLevel.width / 2);
                            var positionY = 1178;
                            nextLevel.position.set(950, 1178);
                            nextLevel.alpha = 0;
                            nextLevel.anchor.set(0.5, 0.5);
                            nextLevel.addTo(self.stage);

                            nextLevel.interactive = true;
                            nextLevel.click = nextLevel.tap = function() {
                                nextLevel.interactive = false;

                                game.LevelsChallenge = new game.ClassLevelsChallenge();
                                game.LevelChallenge.setLevel(self.nextLevelId);

                                var tween = new game.Tween(nextLevel.scale)
                                        .to({x: 0.75, y: 0.75}, 200)
                                        .repeat(1)
                                        .yoyo()
                                        .easing(game.Tween.Easing.Quadratic.Out)
                                        .onComplete(function () {
                                            nextLevel.interactive = true;
                                            game.system.setScene('GameChallenge');
                                        });

                                // Google Analytics
                                if (navigator.isCocoonJS) {
                                    game.Analytics.ga.trackEvent('Game actions', 'Next level (Challenge)');
                                }

                                game.audio.playSound('button-click');
                                tween.start();

                            };

                            var tweenNext1 = new game.Tween(nextLevel)
                                    .to({alpha: 1}, 700)
                                    .easing(game.Tween.Easing.Quadratic.Out).start();

                            var tweenNext2 = new game.Tween(nextLevel.position)
                                    .to({x: positionX, y: positionY}, 600)
                                    .easing(game.Tween.Easing.Quadratic.Out).start();
                        }
                    });


            var values = {};
            values.points = this.score.totalPointsOld;

            if (this.score.levelPoints > 0) {
                var tween = new game.Tween(values)
                        .to({points: this.score.totalPoints}, 1000)
                        .delay(400)
                        .easing(game.Tween.Easing.Cubic.Out)
                        .onUpdate(function () {
                            var points = Math.round(values.points);
                            textTotalPoints.setText('Total score: ' + points.toString());
                        })
                        .onComplete(function () {
                            game.audio.playSound('score-success');
                            tweenSuccess1.start();
                            tweenSuccess2.start();
                        })
                        .start();
            } else {
                game.audio.playSound('score-success');
                tweenSuccess1.start();
                tweenSuccess2.start();
            }

        },


    });




});
