game.module(
    'game.scenes.levelInfoTraining'
);

game.body(function() {

    game.createScene('LevelInfoTraining', {
        backgroundColor: 0xb9bec7,

        init: function() {
            // Google Analytics
            if (navigator.isCocoonJS) {
               game.Analytics.ga.trackView('Level Info Menu (Training)');
            }

            this.addBackground();
            this.addHud();
            this.showBestScore();
        },

        addBackground: function () {
            var background = new game.Sprite('images/game-background.png');
            background.position.set((game.system.width / 2), 0);
            background.anchor.set(0.5, 0);
            background.addTo(this.stage);
        },

        addHud: function() {
            var menuBack = new game.Sprite('images/btn-menu-back.png');
            menuBack.position.set(((game.system.width/2) - 330 + (menuBack.width / 2)), 1180);
            menuBack.anchor.set(0.5, 0.5);
            menuBack.addTo(this.stage);

            menuBack.interactive = true;
            menuBack.click = menuBack.tap = function() {
                var tween = new game.Tween(menuBack.scale)
                        .to({x: 0.75, y: 0.75}, 200)
                        .repeat(1)
                        .yoyo()
                        .easing(game.Tween.Easing.Quadratic.Out)
                        .onComplete(function () {
                            game.system.setScene('LevelSelectTraining');
                        })
                        .start();
            };


            var playLevel = new game.Sprite('images/btn-play-level.png');
            playLevel.position.set(((game.system.width/2) + 330 - (playLevel.width / 2)), 1180);
            playLevel.anchor.set(0.5, 0.5);
            playLevel.addTo(this.stage);

            playLevel.interactive = true;
            playLevel.click = playLevel.tap = function() {

                var tween = new game.Tween(playLevel.scale)
                        .to({x: 0.75, y: 0.75}, 200)
                        .repeat(1)
                        .yoyo()
                        .easing(game.Tween.Easing.Quadratic.Out)
                        .onComplete(function () {
                            game.system.setScene('GameTraining');
                        })
                        .start();
            };

        },

        convertTime: function (time, zero) {
            // prevod casu na minuty a vteriny
            // zero tru pokud pridat nuly jinak false
            var min = 0;
            var sec = 0;
            var ms = 0;

            if (time > 60000) {
                min = Math.floor(time / 60000);
                rest = time % 60000;
                if (rest > 0) {
                    sec = Math.ceil(rest / 1000);
                }
            } else {
                min = 0;
                sec = Math.ceil(time / 1000);
            }

            if (sec <= 9 && zero == true) {
                sec = '0' + sec;
            }

            var result = {};

            result.min = min.toString();
            result.sec = sec.toString();

            return result;
        },

        showBestScore: function () {

            var title = 'Level ' + game.Level.levelNumber + '';
            var textTitle = new game.BitmapText(title.toString(), {font: '60px score-text'});
            textTitle.position.set( ((game.system.width / 2) - (textTitle.width / 2)), 30);
            textTitle.addTo(this.stage);


            var patternsNumber = game.Level.patternNumber;
            var textPatterns = new game.BitmapText('Patterns: ' + patternsNumber.toString(), {font: '40px score-text'});
            textPatterns.position.set( ((game.system.width / 2) + 50), 120);
            textPatterns.alpha = 0;
            textPatterns.addTo(this.stage);

            var tweenPatterns = new game.Tween(textPatterns)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();


            var result = this.convertTime(game.Level.levelTime);
            var time = result.min + ':' + result.sec;
            var textTime = new game.BitmapText('Time limit: ' + time.toString(), {font: '40px score-text'});
            textTime.position.set( ((game.system.width / 2) - 330), 120);
            textTime.alpha = 0;
            textTime.addTo(this.stage);

            var tweenTime = new game.Tween(textTime)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();



            var title2 = 'Best';
            var textTitle2 = new game.BitmapText(title2.toString(), {font: '50px score-text'});
            textTitle2.position.set( ((game.system.width / 2) - (textTitle2.width / 2)), 200);
            textTitle2.addTo(this.stage);


            var storageKeyTime = 'level' + game.Level.levelNumber + 'Time';
            if (game.storage.has(storageKeyTime)) {
                var displayTime = game.storage.get(storageKeyTime);
            } else {
                var displayTime = 0;
            }
            var textBestTime = new game.BitmapText('Time: ' + displayTime.toString(), {font: '40px score-text'});
            textBestTime.position.set( ((game.system.width / 2) + 50), 290);
            textBestTime.alpha = 0;
            textBestTime.addTo(this.stage);

            var tweenBestTime = new game.Tween(textBestTime)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();


            var storageKeyHighScore = 'level' + game.Level.levelNumber + 'HighScore';
            if (game.storage.has(storageKeyHighScore)) {
                var highScore = game.storage.get(storageKeyHighScore);
            } else {
                var highScore = 0;
            }
            var textHighScore = new game.BitmapText('HighScore: ' + highScore.toString(), {font: '40px score-text'});
            textHighScore.position.set( ((game.system.width / 2) - 330), 290);
            textHighScore.alpha = 0;
            textHighScore.addTo(this.stage);

            var tweenHighScore = new game.Tween(textHighScore)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();


            var storageKeyMistakes = 'level' + game.Level.levelNumber + 'Mistakes';
            if (game.storage.has(storageKeyMistakes)) {
                var mistakes = game.storage.get(storageKeyMistakes);
            } else {
                var mistakes = 0;
            }
            var textMistakes = new game.BitmapText('Mistakes: ' + mistakes.toString(), {font: '40px score-text'});
            textMistakes.position.set( ((game.system.width / 2) + 50), 350);
            textMistakes.alpha = 0;
            textMistakes.addTo(this.stage);

            var tweenMistakes = new game.Tween(textMistakes)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();


            var storageKeySuccess = 'level' + game.Level.levelNumber + 'Success';
            if (game.storage.has(storageKeySuccess)) {
                var success = game.storage.get(storageKeySuccess);
            } else {
                var success = 0;
            }
            var textSuccess = new game.BitmapText('Rating: ' + success.toString() + '%', {font: '40px score-text'});
            textSuccess.position.set( ((game.system.width / 2) - 330), 350);
            textSuccess.alpha = 0;
            textSuccess.addTo(this.stage);

            var tweenSuccess = new game.Tween(textSuccess)
                    .to({alpha: 1}, 400)
                    .easing(game.Tween.Easing.Quadratic.Out).start();



        },



    });


});
