game.module(
    'game.scenes.levelSelectTraining'
);

game.body(function() {

    game.createScene('LevelSelectTraining', {
        backgroundColor: 0xb9bec7,
        levels: [],
        pages: [],      // pole pro kontainer stranek
        pageNumber: 1,      // kolik je stranek
        actualPage: 0,
        btnPagePrev: {},
        btnPageNext: {},

        init: function() {
            // Google Analytics
            if (navigator.isCocoonJS) {
               game.Analytics.ga.trackView('Level Select Menu (Training)');
            }

            game.LevelsTraining = new game.ClassLevelsTraining();

            this.addBackground();
            this.addHud();
            this.addLevels();
        },

        addBackground: function () {
            var background = new game.Sprite('images/game-background.png');
            background.position.set((game.system.width / 2), 0);
            background.anchor.set(0.5, 0);
            background.addTo(this.stage);
        },

        addHud: function() {
            var self = this;
            var menuBack = new game.Sprite('images/btn-menu-back.png');
            menuBack.position.set(((game.system.width/2) - 330 + (menuBack.width / 2)), 1178);
            menuBack.anchor.set(0.5, 0.5);
            menuBack.addTo(this.stage);

            menuBack.interactive = true;
            menuBack.click = menuBack.tap = function() {
                menuBack.interactive = false;
                game.audio.playSound('button-click');
                var tween = new game.Tween(menuBack.scale)
                        .to({x: 0.75, y: 0.75}, 200)
                        .repeat(1)
                        .yoyo()
                        .easing(game.Tween.Easing.Quadratic.Out)
                        .onComplete(function () {
                            game.system.setScene('MainMenu');
                        })
                        .start();
            };


            this.btnPagePrev = new game.Sprite('images/btn-page-prev.png');
            this.btnPagePrev.position.set(((game.system.width/2) - 330 + (this.btnPagePrev.width / 2)), 1038);
            this.btnPagePrev.anchor.set(0.5, 0.5);
            this.btnPagePrev.visible = false;
            this.btnPagePrev.alpha = 0;
            this.btnPagePrev.addTo(this.stage);

            this.btnPagePrev.interactive = true;
            this.btnPagePrev.click = this.btnPagePrev.tap = function() {
                self.btnPagePrev.interactive = false;
                game.audio.playSound('button-click');
                var tween = new game.Tween(self.btnPagePrev.scale)
                .to({x: 0.75, y: 0.75}, 200)
                .repeat(1)
                .yoyo()
                .easing(game.Tween.Easing.Quadratic.Out)
                .onStart(function () {
                    var oldPage = self.actualPage;
                    self.actualPage = self.actualPage - 1;
                    self.showPage(self.actualPage, oldPage, 'prev');
                })
                .start();
            };

            this.btnPageNext = new game.Sprite('images/btn-page-next.png');
            this.btnPageNext.position.set(((game.system.width/2) + 330 - (this.btnPageNext.width / 2)), 1038);
            this.btnPageNext.anchor.set(0.5, 0.5);
            this.btnPageNext.addTo(this.stage);

            this.btnPageNext.interactive = true;
            this.btnPageNext.click = this.btnPageNext.tap = function() {
                self.btnPageNext.interactive = false;
                game.audio.playSound('button-click');
                var tween = new game.Tween(self.btnPageNext.scale)
                .to({x: 0.75, y: 0.75}, 200)
                .repeat(1)
                .yoyo()
                .easing(game.Tween.Easing.Quadratic.Out)
                .onStart(function () {
                    var oldPage = self.actualPage;
                    self.actualPage = self.actualPage + 1;
                    self.showPage(self.actualPage, oldPage, 'next');
                })
                .start();
            };




        },

        addInteractiveForLevels: function(i) {
            var self = this;
            var levelId = i + 1;
            // jen pokud je level odemceny
            if (this.levels[i].unlocked === true || game.Cheats === true) {
                this.levels[i].background.interactive = true;
                this.levels[i].background.click = this.levels[i].background.tap = function() {
                    // Google Analytics
                    if (navigator.isCocoonJS) {
                        game.Analytics.ga.trackEvent('Setup actions', 'Level ' + levelId + ' (Training)');
                    }

                    self.levels[i].background.interactive = false;
                    game.audio.playSound('button-click');

                    game.LevelTraining = new game.ClassLevelTraining();
                    game.LevelTraining.setLevel(levelId);

                    var tween = new game.Tween(self.levels[i].container.scale)
                            .to({x: 0.75, y: 0.75}, 200)
                            .repeat(1)
                            .yoyo()
                            .easing(game.Tween.Easing.Quadratic.Out)
                            .onComplete(function () {
                                game.system.setScene('GameTraining');       // GameTraining
                            })
                            .start();
                };
            } else {
                this.levels[i].background.interactive = false;
            }
        },

        addLevels: function() {

            this.pages[this.pageNumber - 1] = {};
            this.pages[this.pageNumber - 1].container = new game.Container();
            this.pages[this.pageNumber - 1].container.alpha = 0;
            this.pages[this.pageNumber - 1].container.visible = false;

            var positionX = -225;
            var positionY = 110;
            var positionDifX = 225;
            var positionDifY = 156;

            var column = 1;      // urcuje do jakeho sloupce vkladam
            var row = 1;      // urcuje do jakeho radku vkladam
            var levelInPage = 1;

            var i = 0;
            for (var level in game.LevelsTraining.levels) {
                //console.log(game.LevelsTraining.levels[level]);
                if (column > 3) {
                    column = 1;
                    row += 1;
                }

                if (levelInPage > 18) {
                    this.pageNumber += 1;       // zvysi pocet stranek
                    //console.log('this.pageNumber' + this.pageNumber);
                    this.pages[this.pageNumber - 1] = {};
                    this.pages[this.pageNumber - 1].container = new game.Container();
                    this.pages[this.pageNumber - 1].container.alpha = 0;
                    this.pages[this.pageNumber - 1].container.visible = false;
                    levelInPage = 1;
                    row = 1;
                }

                var newPositionX = (game.system.width/2) + (positionX + ((column - 1) * positionDifX) );
                var newPositionY = (positionY + ((row - 1) * positionDifY) );

                this.levels[i] = {};
                this.levels[i].container = new game.Container();
                this.levels[i].container.position.set(newPositionX, newPositionY);

                this.levels[i].background = new game.Sprite('images/level-bg.png');
                //this.levels[i].background.position.set(newPositionX, newPositionY);
                this.levels[i].background.anchor.set(0.5, 0.5);
                this.levels[i].container.addChild(this.levels[i].background);

                var levelId = i + 1;

                var storageKeyUnlocked = 'levelTraining' + levelId + 'Unlocked';
                if (levelId == 1 || game.storage.has(storageKeyUnlocked) || game.Cheats === true) {

                    var storageKeySuccess = 'levelTraining' + levelId + 'Success';
                    var textSuccess;
                    if (game.storage.has(storageKeySuccess)) {
                        textSuccess = game.storage.get(storageKeySuccess) + '%';
                    } else {
                        textSuccess = '0%';
                    }

                    var textName = 'Level ' + levelId.toString();
                    this.levels[i].textName = new game.BitmapText(textName, {font: '32px level-select-number'});
                    this.levels[i].textName.position.set(0 - Math.round((this.levels[i].textName.width / 2)), 0 - Math.round((this.levels[i].textName.height / 2) + 30) );
                    this.levels[i].container.addChild(this.levels[i].textName);

                    this.levels[i].unlocked = true;
                    this.levels[i].textSuccess = new game.BitmapText(textSuccess, {font: '50px level-select-success'});
                    this.levels[i].textSuccess.position.set(0 - Math.round((this.levels[i].textSuccess.width / 2)), 0 - Math.round((this.levels[i].textSuccess.height / 2) - 21) );
                    this.levels[i].container.addChild(this.levels[i].textSuccess);
                } else {
                    this.levels[i].unlocked = false;
                    this.levels[i].padlock = new game.Sprite('images/level-padlock.png');
                    this.levels[i].padlock.position.set(0, 0);
                    this.levels[i].padlock.anchor.set(0.5, 0.5);
                    this.levels[i].container.addChild(this.levels[i].padlock);
                }

                this.pages[this.pageNumber - 1].container.addChild(this.levels[i].container);
                this.pages[this.pageNumber - 1].container.addTo(this.stage);
                //this.levels[i].container.addTo(this.stage);

                this.addInteractiveForLevels(i);

                column += 1;
                i += 1;
                levelInPage += 1;
            }

            this.pages[0].container.visible = true;
            this.pages[0].container.alpha = 1;      // zobrazeni prvni stranky s levely

        },

        showPage: function(pageId, pageOldId, direction) {
            //console.log('number' + number);

            if (direction === 'prev') {
                oldPagePositionX = game.system.width;
                actualPagePositionX = -game.system.width;
            } else {
                oldPagePositionX = -game.system.width;
                actualPagePositionX = game.system.width;
            }

            this.pages[pageId].container.position.x = actualPagePositionX;
            this.pages[pageId].container.visible = true;
            this.pages[pageId].container.alpha = 1;      // zobrazeni prvni stranky s levely

            var tweenGroup = new game.TweenGroup(function() {
                // console.log('TweenGroup complete');
            });

            var tween1 = new game.Tween(this.pages[pageOldId].container.position)
            .to({x: oldPagePositionX}, 400)     // 360 stupnu je 6.28 radianu
            .easing(game.Tween.Easing.Quadratic.Out)
            .onComplete(function () {
                this.pages[pageOldId].container.visible = false;
                this.pages[pageOldId].container.alpha = 0;      // zobrazeni prvni stranky s levely
            });

            var tween2 = new game.Tween(this.pages[pageId].container.position)
            .to({x: 0}, 400)     // 360 stupnu je 6.28 radianu
            .easing(game.Tween.Easing.Quadratic.Out)
            .onComplete(function () {

            });

            tweenGroup.add(tween1);
            tweenGroup.add(tween2);
            tweenGroup.start();



            // schovani tlacitek
            if (pageId < 1) {
                this.btnPagePrev.interactive = false;
                this.btnPagePrev.visible = false;
                this.btnPagePrev.alpha = 0;
            }
            if (pageId > 0) {
                this.btnPagePrev.interactive = true;
                this.btnPagePrev.visible = true;
                this.btnPagePrev.alpha = 1;
            }
            if (pageId + 1 == this.pageNumber) {
                this.btnPageNext.interactive = false;
                this.btnPageNext.visible = false;
                this.btnPageNext.alpha = 0;
            }
            if (pageId < 1 && pageId + 1 < this.pageNumber) {
                this.btnPageNext.interactive = true;
                this.btnPageNext.visible = true;
                this.btnPageNext.alpha = 1;
            }

        },

    });


    game.createClass('ClassLevelTraining', {
        name: '',
        levelNumber: '',

        init: function() {

        },

        setLevel: function(levelId) {
            // nastavi data zvoleneho levelu z objektu leveluu
            //console.log('level selected: ' + levelId);

            this.name = 'Level ' + levelId;
            this.levelNumber = levelId;
            this.patternNumber = game.LevelsTraining.levels['level' + levelId].patternNumber;
            this.patternSpeed = (401 - (levelId * 1));        //game.LevelsTraining.levels['level' + levelId].patternSpeed;
            this.clickAnimationSpeed = (this.patternSpeed / 1.5) ;
            //this.clickMystakeAnimationSpeed = (this.clickAnimationSpeed * 1.5);
            this.patternDelay =  ((2 * this.patternSpeed) + 301 - (levelId * 1));        //game.LevelsTraining.levels['level' + levelId].patternDelay;
            //this.shapesNumber = game.LevelsTraining.levels['level' + levelId].shapesNumber;
            this.shapeSize = game.LevelsTraining.levels['level' + levelId].shapeSize;
            this.locationSize = game.LevelsTraining.levels['level' + levelId].locationSize;
            this.locationsNumber = game.LevelsTraining.levels['level' + levelId].locations.length;
            this.locations = game.LevelsTraining.levels['level' + levelId].locations;
            this.levelTime = ( (4 + Math.round(this.patternNumber * 3)) * 1000);        // cas v ms
            //this.levelLives = 3;
        },

    });


});
